package RPM::Source::BundleImport::BundleCache;

use strict;
use warnings;
use Carp;

use Source::Bundle::Dir;
use Source::Bundle::Archive 0.002;
use Source::Bundle::Factory 0.001;

our $VERSION=0.002;
our $verbose=0;

sub new {
    my $class = shift;
    my $self = {
	@_
    };
    croak "-bundle_container and -bundle_name is required"
	if not $self->{-bundle_container} and not $self->{-bundle_name};
    bless $self, $class;
    return $self;
}

sub create_from_input_bundle {
    my ($class,$bundle,$name)=@_;
    return $class->new(
	-bundle_container => $bundle,
	-bundle_name => $name || '(unknown)',
	);
}

sub create_from_input_file {
    my ($class,$file)=@_;
    print STDERR "DEBUG: BundleImport::BundleCache: got $file\n" if $verbose > 2;
    return $class->new(
	#-bundle_container => Source::Bundle::Archive->new(-file=>$file),
	-bundle_container => Source::Bundle::Factory->create($file),
	-bundle_name => $file,
	);
}

sub bc_get_input_bundle {
    my ($self)=@_;
    return $self->{-bundle_container};
}

sub bc_get_input_name {
    my ($self)=@_;
    return $self->{-bundle_name};
}

sub bc_get_extracted_bundle {
    my ($self)=@_;
    $self->__check_extracted();
    return $self->{-extracted_bundle};
}
    
sub bc_get_extracted_tmpdir {
    my ($self)=@_;
    $self->__check_extracted();
    return $self->{-extracted_tmpdir};
}

sub __check_extracted {
    my ($self)=@_;
    unless ($self->{-extracted_bundle}) {
	# extract all files: to be used in $analyzer and so on.
	my $extracted_tmpdir=$self->{-bundle_container}->extract();
	$self->{-extracted_tmpdir}=$extracted_tmpdir;
	$self->{-extracted_bundle}=Source::Bundle::Dir->new(-file=>$extracted_tmpdir);
    }
}

sub bc_get_unpack_path {
    my ($self)=@_;
    my $unpack_path=$self->{-unpack_path};
    if (! $unpack_path) {
	$unpack_path=$self->bc_get_extracted_tmpdir.'/'.
	    $self->bc_get_input_bundle->depth_prefix;
	$unpack_path=~s!//$!/!;
	$self->{-unpack_path}=$unpack_path;
    };
    return $unpack_path;
}

sub bc_get_rebased_filelist {
    my ($self)=@_;
    my $rebased_filelist=$self->{-rebased_filelist};
    if (! $rebased_filelist) {
	$rebased_filelist=$self->bc_get_input_bundle()->rebase_to_depth();
	$self->{-rebased_filelist}=$rebased_filelist;
    }
    return $rebased_filelist;
}

sub bc_get_rebased_depth1dir {
    my ($self)=@_;
    my $rebased_depth1dir=$self->{-rebased_depth1dir};
    if (! $rebased_depth1dir) {
	$rebased_depth1dir=&__rebased_filelist_to_depth1dir($self->bc_get_rebased_filelist);
	$self->{-rebased_depth1dir}=$rebased_depth1dir;
    }
    return $rebased_depth1dir;
}

sub __rebased_filelist_to_depth1dir {
    my ($filelist)=@_;
    my %depth1dir;
    foreach my $file (@$filelist) {
	$depth1dir{$1}=1 if $file=~m!^([^/]+)/!;
    }
    return \%depth1dir;
}


1;
