package RPM::Source::Transformation::Factory::BundleImport::CopySource;

use strict;
use warnings;
use Carp;
use File::Basename;

use RPM::Source::Transformation::Factory 0.008;
our @ISA=qw/RPM::Source::Transformation::Factory/;
push @RPM::Source::TransformContainer::FACTORY_CLASSES, __PACKAGE__;

our $VERSION=0.002;
our $verbose=0;

sub transformations {
    return RPM::Source::Transformation::OptHandlerWrapper->new(
	# PRIORITY_BUNDLE_SOURCE (ruby) < priority < PRIORITY_PRECONVERT (tarball repack)
	-priority => PRIORITY_BUNDLE_PRECOLLECT,
	-name => 'copy_source_to_SOURCEDIR',
	-description => 'copy source to src.rpm\'s SOURCEDIR',
	-opt_handler_sub=>\&opt_handler);
}

sub opt_handler {
    my (undef,$spec,undef,$converter_opt,$resourceobj)=@_;
    my $file=$converter_opt->{-opt_bundle_source_file};
    $file//=$converter_opt->{-opt_bundle_input_file};
    return RPM::Source::TransformContainer::TRANSFORMATION_FAILURE if not defined($file);
    if (-d $file) {
	print STDERR 'experimental: CopySource: is a directory: ',$file,"\n";
	return RPM::Source::TransformContainer::TRANSFORMATION_OK;
    }
    if (not $spec->{GEARDIR}) {
	my $sourcedir=$spec->_get_sourcedir();
	my $destfile=$sourcedir.'/'.basename($file);
	if ((-e $file) and (not -e $destfile)) {
	    # use actual cp, not opt, because with opt tarball repack (Factory::CommandLine,
	    # PRIORITY_PRECONVERT) will not work as we expect.
	    # push @{$converter_opt->{-opt_copy_to_sources}}, $file;
	    system('cp', $file, $destfile)==0 or die "copy $file, $destfile failed: $!";
	    warn 'DEBUG: CopySource: copied ',$file,' -> ',$destfile,"\n" if $verbose;
	}
    }
    return RPM::Source::TransformContainer::TRANSFORMATION_OK;
}

1;
