package RPM::Source::Transformation::Factory::BundleImport::DistroMap;

use strict;
use warnings;
use Carp;

use Source::Shared::Resource::DistroMap;
use RPM::Source::Transformation::Factory;

our @ISA=qw/RPM::Source::Transformation::Factory/;

push @RPM::Source::TransformContainer::FACTORY_CLASSES, __PACKAGE__;

our $verbose=0;

our $matcher_class;
my $matcher;

sub transformations {
    return RPM::Source::Transformation::OptHandlerWrapper->new(
	-priority => PRIORITY_BUNDLE_SETUP,
	-name => 'distromap metadata name',
	-opt_handler_sub=>\&opt_handler);
}

sub opt_handler {
    my (undef,undef,undef,$converter_opt,$resourceobj)=@_;
    my $distromap=$resourceobj->get_distromap;
    if (not $matcher) {
	die __PACKAGE__, ": Internal error: matcher_class is empty" unless $matcher_class;
	# lazy initialization of matcher
	$matcher=$matcher_class->new();
	# hack; but useful; let it be shared
	$distromap->die_if_no_distromap();
    }
    # set name
    my $bundle_name=$converter_opt->{-opt_bundle_name};
    my $bundle_map=$matcher->origin2dest($bundle_name,$resourceobj);
    die "package $bundle_name is uncomparably mapped to $bundle_map" if $distromap->versions_are_uncomparable($bundle_name);
    # TODO: properly define (prefixator?) (builder_name?) see python(|2|3)-; quick hack is s/builder_name(?:[\d\.]+)?-
    # TODO2: what if we have 2 names (python2 + python3) in $distromap? --- additional option to choose what to create?
    if ($bundle_map ne $RPM::Source::BundleImport::builder_name.'-'.$bundle_name) {
	$converter_opt->{-rename}=$bundle_map;
    }

    return RPM::Source::TransformContainer::TRANSFORMATION_OK;
}


1;
