package RPM::Source::Transformation::Factory::BundleImport::DownloadInput;

use strict;
use warnings;
use Carp;

use Source::Shared::Utils::Downloader qw/cached_download/;
use RPM::Source::Transformation::Factory 0.008;
our @ISA=qw/RPM::Source::Transformation::Factory/;

push @RPM::Source::TransformContainer::FACTORY_CLASSES, __PACKAGE__;

our $VERSION=0.001;
our $verbose=0;
our $download_srpm_cachedir;
our $download_gear_cachedir;

sub register_messages {
    my ($self,$extopt)=@_;
    RPM::Source::Transformation::Component::add_opt($extopt, {
	-opt_bundle_download_url => undef,
	});
}

sub transformations {
    return RPM::Source::Transformation::OptHandlerWrapper->new(
	-priority => PRIORITY_PRE_BUNDLE,
	-name => 'download_source_if_missing',
	-description => 'download source if missing',
	-opt_handler_sub=>\&opt_handler_download);
}

# Optional; Right now some input handlers download, see BundleInput
# others are not, see UpdateList5. opt_handler_download is for UpdateList5.
# TODO: move download from BundleInput here?

sub opt_handler_download {
    my (undef,$spec,undef,$converter_opt,$resourceobj)=@_;
    
    # -----------------------------
    # move options from RPM/Source/BundleInput.pm: our $opt_download_cache_dir='in';
    # 'download-cache-dir' => \$opt_download_cache_dir,
    # -----------------------------

    my $file=$converter_opt->{-opt_bundle_input_file};
    warn 'DEBUG: BundleImport::PrepareSource: before url: file=',$file,"\n" if $verbose>4;
    
    if (!defined($file)) {
	my $url=$converter_opt->{-opt_bundle_download_url};
	$url//=$converter_opt->{-opt_bundle_url};
	if ($url) {
	    my $download_dir;
	    if ($spec->{GEARDIR}) {
		$download_dir=$download_gear_cachedir;
		$download_dir//=$spec->{GEARDIR};
		warn 'DEBUG: gear to ',$download_dir,"\n" if $verbose>1;
	    } else {
		$download_dir=$download_srpm_cachedir;
		$download_dir//=$spec->_get_sourcedir();
		warn 'DEBUG: srpm to ',$download_dir,"\n" if $verbose>1;
	    }
	    warn 'DEBUG: cached_download to ',$download_dir,' url=',$url,"\n" if $verbose>1;
	    $file=cached_download($url,$download_dir);
	    $converter_opt->{-opt_bundle_input_file}=$file;
	}
    }
    return RPM::Source::TransformContainer::TRANSFORMATION_OK;
}

1;
