package RPM::Source::Transformation::Factory::BundleImport::MetadataCollector;

use strict;
use warnings;
use Carp;

use RPM::Source::BundleImport::Helpers;
use RPM::Source::BundleImport::Metadata;
use RPM::Source::Transformation::Resource::InputBundle;
use RPM::Source::Transformation::Resource::MetadataCollector;
use RPM::Source::Transformation::Factory 0.009;

our @ISA=qw/RPM::Source::Transformation::Factory/;
push @RPM::Source::TransformContainer::FACTORY_CLASSES, __PACKAGE__;

our $verbose=0;

my $opt_collect_from_source_tree=1;
our $opt_collect_from_parent=0;
our $opt_force_parent_summary_description=0;

sub transformations {
    my @transformations=
	RPM::Source::Transformation::OptHandlerWrapper->new(
	    -priority => PRIORITY_BUNDLE_POSTCOLLECT,
	    -name => 'metadata_collector_from_parent',
	    -opt_handler_sub=>\&opt_handler_metadata_collector_from_parent);
    push @transformations,
	RPM::Source::Transformation::OptHandlerWrapper->new(
	-priority => PRIORITY_BUNDLE_POSTCOLLECT,
	-name => 'metadata_collector_from_source_tree',
	-opt_handler_sub=>\&opt_handler_metadata_collector_from_source_tree)
	if $opt_collect_from_source_tree;
    return @transformations;
}

sub opt_handler_metadata_collector_from_parent {
    my (undef,undef,$parent,$converter_opt,$resourceobj)=@_;
    return RPM::Source::TransformContainer::TRANSFORMATION_OK if not $parent;
    return RPM::Source::TransformContainer::TRANSFORMATION_OK
	if not $converter_opt->{-opt_regenerate} and not $opt_collect_from_parent
		and not $opt_force_parent_summary_description;
    
    my $metacollector=$resourceobj->metadata_collector;
    my $mainsec=$parent->main_section;
    my $license=$mainsec->get_tag('License');
    $metacollector->add(
	'license', -priority => MPRIORITY_OLD_PARENT,
	-description => 'use parent license',
	-value => $license) if $license;

    my $summary=$mainsec->get_tag('Summary');
    $metacollector->add('summary', -priority =>
		$opt_force_parent_summary_description ? MPRIORITY_PARENT_OVERRIDE : MPRIORITY_OLD_PARENT,
		-value => $summary,
		-description => 'use parent summary',
	    ) if $summary and $summary ne 'unknown';

    my $descrsec=$parent->get_section('description','');
    if ($descrsec) {
	my @tail=@{$descrsec->get_bodyref};
	&RPM::Source::SpecSection::__shift_header_from_a_body_array($descrsec,\@tail);
	while (@tail>0 and $tail[$#tail]=~/^\s*(?:#|\%(?:if|else|endif)|$)/) {
	    pop @tail;
	}
	my $description;
	$description=join('',@tail) if @tail;
	if ($description) {
	    $description=~s/[\s\n]*$//s;
	    $metacollector->add('description', -priority =>
		$opt_force_parent_summary_description ? MPRIORITY_PARENT_OVERRIDE : MPRIORITY_OLD_PARENT,
		-value => $description,
		-description => 'use parent description',
	    ) if $description ne '%summary' and $description ne '%{summary}' and $description !~ '^From summary:';
	}
    }
    
    return RPM::Source::TransformContainer::TRANSFORMATION_OK;
}

sub opt_handler_metadata_collector_from_source_tree {
    my (undef,undef,undef,$converter_opt,$resourceobj)=@_;
    my $metacollector=$resourceobj->metadata_collector;

    my $unpack_path=$resourceobj->bundle_import_unpack_path();
    my $filelist=$resourceobj->bundle_import_rebased_filelist();
    #my $depth1dir=$resourceobj->bundle_import_rebased_depth1dir();

    ## TODO warn 'see LICENSE'
    $metacollector->add(
	'license', -priority => MPRIORITY_GUESS,
	-value => &_SourceBundleImport::helper::guess_license($unpack_path,$filelist),
	-description => 'guess_license_from_filelist',
	) if $metacollector->want('license',MPRIORITY_GUESS);

    if ($metacollector->want('summary',MPRIORITY_GUESS)) {
	my $summary=&_SourceBundleImport::helper::guess_summary_from_pc_in($unpack_path,$filelist);
	$metacollector->add('summary', -priority => MPRIORITY_GUESS,
		-value => $summary,
		-description => 'guess_summary_from_pc_in',
	    );
    }

    if ($metacollector->want('summary',MPRIORITY_GUESS) or $metacollector->want('description',MPRIORITY_GUESS)) {
	foreach my $readme (@_SourceBundleImport::helper::KNOWN_README) {
	    my $readme_f="$unpack_path/$readme";
	    if (-s $readme_f) {
		my ($summary,$description)=&_SourceBundleImport::helper::guess_summary_description_from_README($readme_f);
		$metacollector->add('summary', -priority => MPRIORITY_GUESS,
			-description => 'guess_summary_from_README='.$readme,
			-value => $summary) if $summary;
		$metacollector->add('description', -priority => MPRIORITY_GUESS,
			-description => 'guess_description_from_README='.$readme,
			-value => $description) if $description;
		last if (!$metacollector->want('summary',MPRIORITY_GUESS) and !$metacollector->want('description',MPRIORITY_GUESS));
	    }
	}
    }
    return RPM::Source::TransformContainer::TRANSFORMATION_OK;
}

1;
